#include "CController.h"
#include "FKeyboard.h"
#include "Entity.h"

CController::CController()
{
	m_typeID = ID::COMPONENT::CONTROLLER;
	m_isPaused = false;
}

CController::~CController()
{
	m_CommandMap.clear();
}

CController* CController::addComponentTo(Entity* object, map<int, int> commandMap)
{
	auto component = new CController();
	component->m_owner = object;

	component->m_CommandMap.swap(commandMap);

	object->addComponent(component);
	return component;
}

CController* CController::addComponentTo(Entity* object)
{
	map<int, int> virtualKeymap;
	return addComponentTo(object, virtualKeymap);
}

int CController::commandProcess(int command)
{
	switch (command)
	{
	case COMMAND::PAUSE:
		m_isPaused = true;
		break;

	case COMMAND::RESUME:
		m_isPaused = false;
		break;

	default:
		return FERROR_INVALID_COMMAND;
	}

	return FSUCCESS;
}

int CController::update(float deltaTime)
{
	CBase::update(deltaTime);

	if (!m_isPaused)
		for each (Keycode_Command_Pair commandPair in m_CommandMap)
			if (FKeyboard::isKeyDown(commandPair.first))
				m_owner->pushCommand((Entity::COMMAND)commandPair.second);

	return FSUCCESS;
}

int CController::addKey(int keyCode, int command)
{
	m_CommandMap.erase(keyCode);
	m_CommandMap.insert(Keycode_Command_Pair(keyCode, command));

	return FSUCCESS;
}

int CController::eraseKey(int keyCode)
{
	m_CommandMap.erase(keyCode);

	return FSUCCESS;
}

ComponentProperty CController::getProperty(int propertyFlag)
{
	if (propertyFlag == PropertyFlag::COMMAND_MAP)
	{
		return ComponentProperty(&m_CommandMap, ComponentProperty::PTYPE_COMMAND_MAP);
	}
}

int CController::setProperty(ComponentProperty propertyData, int propertyFlag) throw(ComponentProperty::InvalidPointerType)
{
	if (propertyFlag == PropertyFlag::COMMAND_MAP)
	{
		if (propertyData.ptrCommandPair->second == CBase::COMMAND::NULL_COMMAND)
			eraseKey(propertyData.ptrCommandPair->first);
		else
			addKey(	propertyData.ptrCommandPair->first,
					propertyData.ptrCommandPair->second);
	}

	return FSUCCESS;
}
