#ifndef __ENTITY_H__
#define __ENTITY_H__

#include <map>
#include <vector>
#include <windows.h>
#include "CBase.h"
#include "Config.h"

using namespace std;

class Entity
{
	typedef pair<ID::COMPONENT, CBase*> Pair_ID_ptrCBase;
public:
	typedef void(*EntityReactFunction)(Entity* reactor);
	enum COMMAND
	{
		NULL_COMMAND = -1,
		HIDE,
		SHOW
	};

	struct ReactCommand
	{
		Entity::COMMAND command;
		ID::COMPONENT	componentID;
		int				reactCommand;

		// @param _command: The Entity Command cause this react.
		// @param _componentID: The Component ID mark the component get react.
		// @param _reactCommand: The given command.
		ReactCommand(Entity::COMMAND _command, ID::COMPONENT _componentID, int _reactCommand)
		{
			command = _command;
			componentID = _componentID;
			reactCommand = _reactCommand;
		}
	};

	struct ReactFunction
	{
		Entity::COMMAND command;
		EntityReactFunction	reactFunction;

		// @param _command: The Entity Command cause this react.
		// @param _reactCommand: The .
		ReactFunction(Entity::COMMAND _command, EntityReactFunction _reactFunction)
		{
			command = _command;
			reactFunction = _reactFunction;
		}
	};

	Entity();
	~Entity();

	int update(float delta);
	void draw(HDC hdc);
	void addComponent(CBase* component);

	// To get property from the component owned by this entity.
	// @param componentFlag: the flag define the component you gonna access.
	// @param propertyFlag: the flag define the property you wanna get.
	ComponentProperty getProperty(ID::COMPONENT componentFlag, int propertyFlag) 
		throw(ID::Unexpected);

	// To set the component property.
	// @param componentFlag: the flag define the component you gonna access.
	// @param propertyData: the variable contain pointer to data location.
	// @param propertyFlag: the flag define the property you wanna set.
	int setProperty(ID::COMPONENT componentFlag, ComponentProperty propertyData, int propertyFlag) 
		throw(ID::Unexpected, ComponentProperty::InvalidPointerType);

	// To push an Entity Command into command queue.
	// Possible pushed command are listed in Entity::COMMAND.
	// The commands are orderly.
	// @param command: the command to be pushed in command queue.
	int pushCommand(Entity::COMMAND command);

	// To get the next command in queue.
	// @return: the popped command. NULL_COMMAND which mean the queue was empty.
	Entity::COMMAND popCommand();

	// To process the received Entity Command.
	// @param: the processing command.
	int commandProcess(Entity::COMMAND command);

	// To add new react to this Entity.
	// IMPORTANT: The reacts are orderly and the ReactFunction(s) are called after ReactCommand(s).
	// @param react: the new pushed react.
	int settingReact(ReactCommand react);
	int settingReact(ReactFunction react);

private:
	map<ID::COMPONENT, CBase*> m_components;
	vector<Entity::COMMAND>	m_commandQueue;
	vector<ReactCommand>	m_reactCommands;
	vector<ReactFunction>	m_reactFunctions;
};

#endif // __ENTITY_H__
